
#include "TeamComputer.h"

#include <iomanip>

#define MAX_SEARCH_ITERATIONS 500
#define MIN_SEARCH_ITERATIONS 50


int Solution::numPlayers=0;
int Solution::goalTeamValue = 0;
int Solution::acceptableTeamValueVariance=0;


/*
bool playerValueGreater(const string& p1, const string& p2)
{
return playerValues[p1] > playerValues[p2];
}
*/


/*
Does a depth-first search looking for a complete team with a value that's acceptable.
It'll stop when it finds a team composition that works, searches the entire space, 
or hits the MAX_SEARCH_ITERATIONS count.
*/
void TeamComputer::doOptimalTeamSearch(Solution current, Solution& bestSoFar, bool& done) {
	searchCount++;

	// bail if we've searched enuff
	if(searchCount > MAX_SEARCH_ITERATIONS && bestSoFar.getDelta() != INT_MAX) {
		done =true;
		return;
	}

	Solution next;
	for(int i = (int)current.getUnassignedPlayers().size()-1; i >= 0; i--) {
		next = current;
		next.assignPlayer(current.getUnassignedPlayers()[i]);
		if(next.isComplete()) {
			if(next > bestSoFar) {
				bestSoFar = next;
				if(bestSoFar.isWithinAcceptableValue() && searchCount > MIN_SEARCH_ITERATIONS) {
					done = true;
				}
			}
			return;
		}
		if(! done) {
			doOptimalTeamSearch(next, bestSoFar, done);
		}
	}
}


// puts the recommended team member names into team1 and team2
void TeamComputer::getEvenTeams(strvec& team1Players, strvec& team2Players)
{

	int totalPlayerValue = 0;

	// clear old results 
	team1.reset();
	team2.reset();

	// setup the total value of all present players
	for(playerlist::const_iterator i = players.begin(); i != players.end(); i++) {
		totalPlayerValue += i->skill;
	}

	// make a list of all the players without teams
	playervec unassignedPlayers;
	for(playerlist::const_iterator i = players.begin(); i != players.end(); i++) {
		 if(i->team == MARINE_TEAM) {
			team1.addPlayer(*i);
		} else if(i->team == ALIEN_TEAM) {
			team2.addPlayer(*i);
		} else { // this'll pull spectators as well as people in the ready room
			unassignedPlayers.push_back(*i);
		}
	}

	// if there're an odd number of players, add a dummy player
	if(numPlayers %2 == 1) {
		numPlayers++;
		unassignedPlayers.push_back(PlayerInfo(DUMMY_PLAYER_NAME,0));
	}

	// setup a few variables used for the search
	Solution::goalTeamValue = totalPlayerValue / 2;
	Solution::acceptableTeamValueVariance =  static_cast<int>(totalPlayerValue * .01); // +/- 10% should be ok, eh?
	Solution::numPlayers = numPlayers;


	//
	// Either sort of shuffle them.  If the player names are randomly suffled, 
	// it should help ensure fairly random teams.  I was initially going
	// to sort them thinking it would make finding a solution easier, but
	// the random thing works fine.
	//

	//	sort(unassignedPlayers.begin(), unassignedPlayers.end(), playerValueGreater );
	random_shuffle(unassignedPlayers.begin(), unassignedPlayers.end() );

	// If players were sorted, I was going to randomly assign 2 ppl to
	// make the teams a little random
	//
	// if at least 6 players, randomly pick 2 for each team to start with
	/*	if(numPlayers >= 6) {
	int r = rand() % unassignedPlayers.size();
	team1.addPlayer(unassignedPlayers[r]);
	unassignedPlayers.erase(unassignedPlayers.begin() + r);

	r = rand() % unassignedPlayers.size();
	team1.addPlayer(unassignedPlayers[r]);
	unassignedPlayers.erase(unassignedPlayers.begin() + r);

	r = rand() % unassignedPlayers.size();
	team2.addPlayer(unassignedPlayers[r]);
	unassignedPlayers.erase(unassignedPlayers.begin() + r);

	r = rand() % unassignedPlayers.size();
	team2.addPlayer(unassignedPlayers[r]);
	unassignedPlayers.erase(unassignedPlayers.begin() + r);
	}
	*/


	Solution inital(team1, unassignedPlayers); // initial base state
	Solution bestSolution; // best team will end up here when search is done
	bool done = false;
	
	// reset search counter
	searchCount = 0;

	// do the search
	doOptimalTeamSearch(inital, bestSolution,done);

	// get the results
	team1 = bestSolution.getTeam();
	
	
	// remove the dummy player from selected team and/or the unassigned players list
	// The dummy player will only exist if there was initially an uneven number of players
	playervec::iterator dummy = find(team1.getPlayers().begin(), team1.getPlayers().end(), DUMMY_PLAYER_NAME);

	if(dummy != team1.getPlayers().end()) {
		team1.getPlayers().erase(dummy);
	} else {
		dummy = find(unassignedPlayers.begin(), unassignedPlayers.end(), DUMMY_PLAYER_NAME);
		if(dummy != unassignedPlayers.end()) {
			unassignedPlayers.erase(dummy);
		}
	}

	// create the player list for the 2nd team (it's anyone not on the first team)
	for(playervec::iterator i = bestSolution.getUnassignedPlayers().begin();
		i != bestSolution.getUnassignedPlayers().end();
		i++)
	{
		team2.addPlayer(*i);
	}


	team1Players = team1.getPlayerNames();
	team2Players = team2.getPlayerNames();

	// cout << "delta is " << bestSolution.getDelta() << "/" << Solution::acceptableTeamValueVariance << endl;
}

// helper func used by TeamComputer::printTeams
void setIntWidth(int& intWidth, int val)
{
	ostringstream oss;
	oss << val;
	if(oss.str().size() > intWidth) intWidth = oss.str().size();
}

// used for sorting player listings by player value in the printTeams function
bool teamsort(const PlayerInfo& a, const PlayerInfo& b)  {
	return a.skill > b.skill;
}

string TeamComputer::printTeams(playervec team1Players, 
								playervec team2Players)
{
	ostringstream oss;
	playervec allPlayers;
	int team1Value = 0;
	int team2Value = 0;
	int pval = 0;

	int marineSkillWidth=0; // used for formatting output (even tho output font is NOT monospaced  grrrrr)
	int alienSkillWidth=0;

	if(team1Players.size() + team2Players.size() == 0 ) {
		return "No players on either team\n";
	}

	for(playervec::const_iterator i = team1Players.begin(); i!= team1Players.end(); i++) {
		team1Value += i->skill;
		setIntWidth(marineSkillWidth, i->skill);
		allPlayers.push_back(*i);

	}
	
	for(playervec::const_iterator i = team2Players.begin(); i!= team2Players.end(); i++) {
		team2Value += i->skill;
		setIntWidth(alienSkillWidth, i->skill);
		allPlayers.push_back(*i);
	}

	ostringstream balanceString;
	balanceString.flags(ios_base::fixed);
	balanceString.precision(2);

	if(team1Value == 0 || team2Value == 0) {
		balanceString << "Teams are a little one sided, eh?\n"; // avoid divide by 0 and add a snappy little response
	} else if(team1Value > team2Value + team2Value*.05) {
		balanceString << "Marines have " << ((float)team1Value / team2Value) << "x skill as aliens\n";
	} else if(team2Value > team1Value + team1Value*.05) {
		balanceString << "Aliens have " << ((float)team2Value / team1Value) << "x skill as marines\n";
	}
	else {
		 balanceString << "Teams are balanced\n";
	}

	sort(team1Players.begin(), team1Players.end(), &teamsort);
	sort(team2Players.begin(), team2Players.end(), &teamsort);
	sort(allPlayers.begin(), allPlayers.end(), &teamsort);

	oss << balanceString.str();
	oss << "\n";
	oss << "+-- Marines\n";
	for(playervec::const_iterator i = team1Players.begin(); i != team1Players.end(); i++ ) {
		oss << "| " << setw(marineSkillWidth) << i->skill << " - " << i->name << "\n";
	}
	oss << "|\n";
	oss << "+-- total: " << team1Value << "\n";
	oss << "\n";

	oss << "+-- Aliens\n";
	for(playervec::const_iterator i = team2Players.begin(); i != team2Players.end(); i++ ) {
		oss << "| " << setw(alienSkillWidth) << i->skill << " - " << i->name << "\n";
	}

	oss << "|\n";
	oss << "+-- total: " << team2Value << "\n";
	oss << "\n";
	oss << "+-- Top " << (allPlayers.size() < 3 ? allPlayers.size() : 3) << " Players\n";

	for(int i = 0; i < 3 && i < allPlayers.size(); i++) {
		PlayerInfo p = allPlayers[i];
		oss << "| #" << i+1 << " " << p.name << " (" << (p.team == MARINE_TEAM ? "marines" : "aliens") << "," << p.skill << ")\n";
	}
	oss << "+--\n";
	oss << "\n";

	if(allPlayers.size() > 1) {
		PlayerInfo p =  allPlayers[allPlayers.size()-1];
		oss << "Worst player: " << p.name << " (" << (p.team == MARINE_TEAM ? "marines" : "aliens") << "," << p.skill << ")\n";
	}


	return oss.str();
}


/* 
int _tmain(int argc, _TCHAR* argv[])
{


strvec playerNames;
playerNames.push_back("dn`indiana jones");
playerNames.push_back("dn`Sin");
playerNames.push_back("e. Pain_User");
playerNames.push_back("Tombo^lerk-practice");
playerNames.push_back("-NSP- The Pope");
playerNames.push_back("retard");
playerNames.push_back("Chick-T[K]R-");
playerNames.push_back("[AG]Webbie-T[K]R-");
playerNames.push_back("Lornath");
playerNames.push_back("I Hate Aliens");
playerNames.push_back("-NSP- Dark Assassin");
playerNames.push_back("mahnsawce");
playerNames.push_back(":+:Uber~PhLuBB:+:");

PlayerValueConfig playerValues;

if(playerValues.isLoaded()) {
TeamComputer evenTeams(playerNames, playerValues);

strvec team1Players, team2Players;

evenTeams.getEvenTeams(team1Players, team2Players);
cout << evenTeams.printTeams();
} else {
cout << "DOH!! teams not loaded\n";
}

return 0;
}
*/
