#ifndef TEAM_COMPUTER_SOLUTION_H_
#define TEAM_COMPUTER_SOLUTION_H_


#include <string>

using namespace std;

#include "PlayerInfo.h"

// Stores a list of players and the current total team value
class TeamInfo {
public:
	TeamInfo() : value(0) {	}

	void addPlayer(const PlayerInfo& player)
	{
		players.push_back(player);
		value += player.skill;
	}

	int getMemberCount()
	{
		return (int)players.size();
	}

	void reset() { players.clear(); value=0; }

	playervec& getPlayers() { return players; }
	

	strvec getPlayerNames() 
	{ 
		strvec playerNames;
		for(playervec::iterator i = players.begin(); i != players.end(); i++) {
			playerNames.push_back(i->name);
		}
		return playerNames; 
	}

	int getValue() { return value; }

private:
	playervec players;
	int value;
};

// A possible "solution" to the search for a balanced team.
// 
class Solution {
public:
	static int goalTeamValue; // ideal value for a solution
	static int acceptableTeamValueVariance; // current value is ok if within this amount of ideal
	static int numPlayers; // total number of players, one team will have 1/2 this number

	Solution() : delta(INT_MAX) { }
	Solution(const TeamInfo& t, const playervec& ps) : team(t), unassignedPlayers(ps)
	{
		delta = team.getValue() - goalTeamValue;
	}


	// do we have enough players?
	bool isComplete() { return team.getPlayers().size() >= (unsigned)numPlayers /2; }

	// adds a player to the team in this solution and removes it from the list of available players
	void assignPlayer(const PlayerInfo& player)
	{
		playervec::iterator pi = find(unassignedPlayers.begin(), unassignedPlayers.end(), player);
		assert(pi != unassignedPlayers.end());
		team.addPlayer(*pi);
		delta = team.getValue() - goalTeamValue;
		unassignedPlayers.erase(pi);
	}


	// is this a viable solution?
	bool isWithinAcceptableValue() { return abs(delta) <= acceptableTeamValueVariance; }

	// determines which solution is more appropriate
	bool operator>(const Solution& sol)
	{
		return abs(delta) < abs(sol.delta);
	}

	//
	// accessors
	//
	TeamInfo& getTeam() { return team; }

	playervec& getUnassignedPlayers() { return unassignedPlayers; }

	int getDelta() { return delta; }

private:
	TeamInfo team;
	playervec unassignedPlayers;
	int delta;
};






#endif