//
// contains bod-specific functions
//


#include <extdll.h>
#include <dllapi.h>
#include <meta_api.h>
#include <cbase.h>


#include "bod.h"

#include <string>
#include <map>
using namespace std;

#include "PlayerValuesConfig.h"



// hopefully more than 32 clients won't connect to the server :)
// I think client indices are 1 based, not 0 based.
const int MAX_CLIENTS=33;

// true if client is alive.
// NOTE: These values are only valid when balanceOnDeath != NO_TEAM
bool PlayerAliveState[MAX_CLIENTS];

// keeps track of each player's team
PlayerInfo PlayerTeam[MAX_CLIENTS];

// true if next person to die on uneven team should be sent to RR
// NOTE: when setting this to a value to true, InitPlayerDeathStatus() must be called
//       so that player death will be properly caught
bool g_balanceOnDeath=false;

float g_imbalanceTime; // when did last imbalance occurr
float g_lastNoticeTime; // when was last notice sent
bool g_balanceNoticePending; // true if balance notice needs to be sent
Team g_assignExtraPlayer; // makes a team appear to have 1 extra player than they really do

void TrackDeaths()
{
	CBaseEntity *pPlayer = NULL;
	// check each client to see if they just died or became alive
	for ( int iIndex = gpGlobals->maxClients; iIndex >= 1 ; iIndex-- ) {
		pPlayer = UTIL_PlayerByIndex( iIndex );	

		// if the player is valid, and...
		if(pPlayer && pPlayer->pev) {

			// ...if the player is on the team we're trying to balance...
			if(pPlayer->pev->team == TeamBalance::get()) {
			
				// ... and the player just died but used to be alive
				if(pPlayer->pev->deadflag != DEAD_NO && PlayerAliveState[iIndex]) {

					// do one last check so we don't balance someone unnecessarily
					TeamBalance::update();
					
					// If there's no need to balance, don't print anything 'cause
					// the teams prolly evened out a while ago, and return w/o doing anything
					if(TeamBalance::get() == NO_TEAM) {
						RETURN_META(MRES_IGNORED);
					}
						
					// this if stmt should be unnecessary, but it can't hurt
					if(g_balanceOnDeath) {
						// send'im to the ready room!
						CLIENT_COMMAND( pPlayer->edict(), "readyroom\n" );
						char buf[256];
						SNPRINTF(buf, 255, "%s died -- sent to the Ready Room for balance\n", STRING( pPlayer->pev->netname ));
						buf[255] = '\0';
						NSASayAll(buf, HUD_PRINTTALK);
					}

					// there may be a need for further balancing (if it was like 6 on 10 or something)
					// balanceOnDeath = UTIL_CheckTeamBalance();  <-- this doesn't reflect the client change
					// It'll be reset anyway by the ClientUserInfoChanged callback.

					// InitPlayerDeathStatus(); <-- not needed, since we'll continue tracking alive/dead status

					// set to dead so we don't try and move this player again until it joins up and becomes alive
					PlayerAliveState[iIndex]=false;
				} 
				// if the player is alive, but used to be dead, mark as alive
				else if(pPlayer->pev->deadflag == DEAD_NO && !PlayerAliveState[iIndex])
				{
					PlayerAliveState[iIndex]=true;
				}
			}
		}
	}

	/* 
	 * This was used to track player dead/alive each and every frame.
	 * The new loop above only tracks when balanceOnDeath is set, and only
	 * tracks deaths on the team that might get balanced.
	 *  ** NOTE ** this code is sorta old and may not work properly if uncommented
	 * 
	for ( int iIndex = gpGlobals->maxClients; iIndex >= 1 ; iIndex-- )
	{
		pPlayer = UTIL_PlayerByIndex( iIndex );	
		if(pPlayer)
		{	
			if(pPlayer->pev && pPlayer->pev->deadflag == DEAD_NO) {
				// the client isn't dead, but it used to be
				if(! PlayerAliveState[iIndex-1]) {
					//UTIL_LogPrintf("client %s born\n", STRING( pPlayer->pev->netname ));
					//UTIL_LogPrintf("client %d born\n", iIndex);
					PlayerAliveState[iIndex-1]=true;
				}
			} else {
				// the client is dead, and it used to be alive
				if(PlayerAliveState[iIndex-1]) {
					//UTIL_LogPrintf("client %s died\n", STRING( pPlayer->pev->netname ));
					//UTIL_LogPrintf("client %d died\n", iIndex);
					PlayerAliveState[iIndex-1]=false;
					
					//
					// so we're supposed to balance the teams
					//
					if(balanceOnDeath != NO_TEAM) {
						// make sure this dude's on the team we want to balance!
						if(pPlayer->pev->team == balanceOnDeath) {
							// send'im to the ready room!
							CLIENT_COMMAND( pPlayer->edict(), "readyroom\n" );
							char buf[256];
							_snprintf(buf, 255, "Sending %s to the Ready Room due to team imbalance", STRING( pPlayer->pev->netname ));
							buf[255] = '\0';
							NSASayAll(buf, HUD_PRINTTALK);

							balanceOnDeath = NO_TEAM;
						}
					}
				}
			}
		}
	}
	*/
	RETURN_META(MRES_IGNORED);
}

PlayerInfo GetLastMarine()
{
	int latestIndex = -1;
	float latestTime = -1;
	CBaseEntity* pPlayer;

	for(int i=gpGlobals->maxClients; i >= 1 ; i--) {
		if(PlayerTeam[i].team == MARINE_TEAM && PlayerTeam[i].joinTime > latestTime) {
			if( (pPlayer = UTIL_PlayerByName(PlayerTeam[i].name.c_str())) && pPlayer->pev->team == MARINE_TEAM) {
				latestIndex = i;
				latestTime = PlayerTeam[i].joinTime;
			}
		}
	}

	return latestIndex == -1 ? PlayerInfo() : PlayerTeam[latestIndex];
}

PlayerInfo GetLastAlien()
{
	int latestIndex = -1;
	float latestTime = -1;
	CBaseEntity* pPlayer;

	for(int i=gpGlobals->maxClients; i >= 1 ; i--) {
		if(PlayerTeam[i].team == ALIEN_TEAM && PlayerTeam[i].joinTime > latestTime) {
			if( (pPlayer = UTIL_PlayerByName(PlayerTeam[i].name.c_str())) && pPlayer->pev->team == ALIEN_TEAM) {
				latestIndex = i;
				latestTime = PlayerTeam[i].joinTime;
			}
		}
	}

	return latestIndex == -1 ? PlayerInfo() : PlayerTeam[latestIndex];
}

void InitPlayerDeathStatus()
{
	CBaseEntity* pPlayer=NULL;

	for ( int iIndex = gpGlobals->maxClients; iIndex >= 1 ; iIndex-- ) {
		pPlayer = UTIL_PlayerByIndex( iIndex );	

		if(pPlayer && pPlayer->pev) {
			PlayerAliveState[iIndex] = pPlayer->pev->deadflag == DEAD_NO;
		} 
	}
}

void bod_init()
{
	CVAR_SET_STRING("mm_bod_version", BOD_VERSION);
	

	for(int i=0;i<MAX_CLIENTS;i++) {
		PlayerAliveState[i]=true;
		PlayerTeam[i] = PlayerInfo();
	}

	g_balanceOnDeath=false;

	// see if one team should be handicapped, appearing to have one more player than they do
	const char* bod_handicapteam = CVAR_GET_STRING("bod_handicapteam");
	if(strcmp("marines",bod_handicapteam) == 0 ) {
		g_assignExtraPlayer = MARINE_TEAM;
	} else if (strcmp("aliens",bod_handicapteam) == 0 ) {
		g_assignExtraPlayer = ALIEN_TEAM;
	} else {
		g_assignExtraPlayer = NO_TEAM;
	}

	string dllfile = GET_PLUGIN_PATH(PLID);
	PlayerValuesConfig::getInst().loadConfig(dllfile.substr(0,dllfile.find_last_of('/')+1) + CVAR_GET_STRING("bod_playervaluesfile"));


	// setting it to 1 is great for testing, but sometimes I forget and leave it when I 
	// send it off to my pub server :(
	if(UNBALANCED_PLAYER_DIFFERENCE <= 1)
		LOG_MESSAGE(PLID, "!!!!!!\nWARNING: bod_mm unbalanced player differenced is too small\n!!!!!!\n");
}


bool MatchesCommand(const char*cmd)
{
	 // char* slashCmd = (char*)_alloca(strlen(cmd)+2); // stack allocation, but i've never used it before and dunno how reliable it is
	char* slashCmd = new char[strlen(cmd)+2];
	slashCmd[0] = '/';
	strncpy(slashCmd+sizeof(char), cmd, strlen(cmd));
	slashCmd[strlen(cmd)+1] = '\0';

	if(STRICMP(CMD_ARGV(0),cmd)==0)
		;
	else {
		if (STRICMP(CMD_ARGV(0),"say")!=0 && STRICMP(CMD_ARGV(0),"say_team")!=0 ) 
		{
			delete[] slashCmd;
			return false;
		}

		if(STRICMP(CMD_ARGV(1),slashCmd)!=0)
		{
			delete[] slashCmd;
			return false;
		}
	}
	delete[] slashCmd;
	return true;
}



void TrackPlayerTeam(edict_t* pEntity)
{
	CBaseEntity *pPlayer = NULL;

	if ( pEntity && !pEntity->free )
		pPlayer = CBaseEntity::Instance( pEntity );	

	if(!pPlayer) 
		return;

	Team newTeam =  intToTeam(pPlayer->pev->team);
	int entIndex = ENTINDEX(pEntity);

	// if the player switched teams
	if(PlayerTeam[entIndex].team != newTeam) {
		// update the record
		PlayerTeam[entIndex].team = newTeam;
		PlayerTeam[entIndex].joinTime = gpGlobals->time;
		PlayerTeam[entIndex].name =STRING(pPlayer->pev->netname);
	}
}



