//
// Contains all functions related to the dll_api
//

#include <extdll.h>
#include <dllapi.h>
#include <meta_api.h>
#include <cbase.h>
#include "bod.h"
#include "BodCommand.h"

#include "TeamComputer.h"


int Spawn(edict_t *pEntity)
{
	if(FStrEq(STRING(pEntity->v.classname), "worldspawn")) //worldspawn means that everything is just starting up
	{
		bod_init();
		
		RETURN_META_VALUE(MRES_HANDLED, 0);
	}

	RETURN_META_VALUE(MRES_IGNORED, 0);
}

void  StartFrame()
{
	// Send warning notice to admins of recent imbalane
	if(	g_balanceNoticePending
		&& fabs(gpGlobals->time - g_imbalanceTime) > IMBALANCE_NOTICE_DELAY
		&& fabs(gpGlobals->time - g_lastNoticeTime) > MIN_IMBALANCE_NOTICE_REPEAT) {

		TeamBalance::update(); // just double check

		if(TeamBalance::get() == MARINE_TEAM) {	
			UTIL_SendAdminsMessage("ADMIN NOTICE: Too many marines\n");
		} else if(TeamBalance::get() == ALIEN_TEAM) { 
			UTIL_SendAdminsMessage("ADMIN NOTICE: Too many aliens\n");
		}
		g_balanceNoticePending = false;
		g_lastNoticeTime = gpGlobals->time;
	}
	
	// only track deaths if we're going to use the info to do some balancing
	if(g_balanceOnDeath) {
		TrackDeaths();
		return; // meta return may be set in TrackDeaths, don't want to interfere with it later
	} else {
		RETURN_META(MRES_IGNORED); 
	}

	// this should never be reached
}



void ClientCommand( edict_t *pPlayerEdict)
{
	//UTIL_LogPrintf("clientcmd: %s\n",CMD_ARGV(0) );

	
	CBaseEntity *pPlayer = NULL;
	
	if ( pPlayerEdict && !pPlayerEdict->free )
	{
		pPlayer = CBaseEntity::Instance( pPlayerEdict );	
	}

	if(!pPlayer)  {
		RETURN_META(MRES_IGNORED);
	}

	// this function should call RETURN_META
	BodCommand<ClientCommandHelper>(pPlayerEdict, pPlayer);
}


// seems to be called when the player joins/changes teams
void ClientUserInfoChanged( edict_t *pEntity, char *infobuffer )
{
	// things might have evened out
	TeamBalance::update();

	// keep track of who's on which team so we know who the last player(s) to join are
	TrackPlayerTeam(pEntity);

	RETURN_META(MRES_IGNORED);
}

void ClientDisconnect(edict_t *pEntity)
{
	// things might have evened out
	// !!  I dunno if this works?  The client may still be in the server when this is called?
	// !!  It can't hurt tho
	TeamBalance::update();

	RETURN_META(MRES_IGNORED);
}


static DLL_FUNCTIONS gFunctionTable = 
{
	NULL, // pfnGameInit
	Spawn, // pfnSpawn
	NULL, // pfnThink
	NULL, // pfnUse
	NULL, // pfnTouch
	NULL, // pfnBlocked
	NULL, // pfnKeyValue
	NULL, // pfnSave
	NULL, // pfnRestore
	NULL, // pfnSetAbsBox

	NULL, // pfnSaveWriteFields
	NULL, // pfnSaveReadFields

	NULL, // pfnSaveGlobalState
	NULL, // pfnRestoreGlobalState
	NULL, // pfnResetGlobalState

	NULL, // pfnClientConnect
	ClientDisconnect, // pfnClientDisconnect
	NULL, // pfnClientKill
	NULL, // pfnClientPutInServer
	ClientCommand, // pfnClientCommand
	ClientUserInfoChanged, // pfnClientUserInfoChanged
	NULL, // pfnServerActivate
	NULL, // pfnServerDeactivate

	NULL, // pfnPlayerPreThink
	NULL, // pfnPlayerPostThink

	StartFrame, // pfnStartFrame
	NULL, // pfnParmsNewLevel
	NULL, // pfnParmsChangeLevel

	NULL, // pfnGetGameDescription
	NULL, // pfnPlayerCustomization

	NULL, // pfnSpectatorConnect
	NULL, // pfnSpectatorDisconnect
	NULL, // pfnSpectatorThink

	NULL, // pfnSys_Error

	NULL, // pfnPM_Move
	NULL, // pfnPM_Init
	NULL, // pfnPM_FindTextureType

	NULL, // pfnSetupVisibility
	NULL, // pfnUpdateClientData
	NULL, // pfnAddToFullPack
	NULL, // pfnCreateBaseline
	NULL, // pfnRegisterEncoders
	NULL, // pfnGetWeaponData
	NULL, // pfnCmdStart
	NULL, // pfnCmdEnd
	NULL, // pfnConnectionlessPacket
	NULL, // pfnGetHullBounds
	NULL, // pfnCreateInstancedBaselines
	NULL, // pfnInconsistentFile
	NULL, // pfnAllowLagCompensation
};

C_DLLEXPORT int GetEntityAPI2(DLL_FUNCTIONS *pFunctionTable, int *interfaceVersion)
{
	if(!pFunctionTable)
	{
		UTIL_LogPrintf("GetEntityAPI2 called with null pFunctionTable\n");
		return(FALSE);
	}
	else if (*interfaceVersion != INTERFACE_VERSION)
	{
		UTIL_LogPrintf("GetEntityAPI2 version mismatch; requested=%d ours=%d\n",
		 *interfaceVersion, INTERFACE_VERSION);
		*interfaceVersion = INTERFACE_VERSION;
		return(FALSE);
	}

	memcpy(pFunctionTable, &gFunctionTable, sizeof(DLL_FUNCTIONS));
	return(TRUE);
}

