// vi: set ts=4 sw=4 :
// vim: set tw=75 :

// engine_api.cpp - implementation of Half-Life engine functions

/*
* Copyright (c) 2001-2003 Will Day <willday@hpgx.net>
*
*    This file is part of Metamod.
*
*    Metamod is free software; you can redistribute it and/or modify it
*    under the terms of the GNU General Public License as published by the
*    Free Software Foundation; either version 2 of the License, or (at
*    your option) any later version.
*
*    Metamod is distributed in the hope that it will be useful, but
*    WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Metamod; if not, write to the Free Software Foundation,
*    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*    In addition, as a special exception, the author gives permission to
*    link the code of this program with the Half-Life Game Engine ("HL
*    Engine") and Modified Game Libraries ("MODs") developed by Valve,
*    L.L.C ("Valve").  You must obey the GNU General Public License in all
*    respects for all of the code used other than the HL Engine and MODs
*    from Valve.  If you modify this file, you may extend this exception
*    to your version of the file, but you are not obligated to do so.  If
*    you do not wish to do so, delete this exception statement from your
*    version.
*
*/




#include <stdio.h>
#include <stdarg.h>

#include <extdll.h>
#include "bod.h"


/*
*  SOOO, why is nothing happening in this file, you may ask (all engine fn ptrs are NULL).
* Well, I was looking for a good way to determine when the game started so when I sent
* messages to the admins saying that the game was unbalanced, they wouldn't get these
* messages sent before the game had started.  It turns out, there's a much easier way
* - just wait 15 seconds after the teams become unbalanced to send a message.  Duhhh.
* But I didn't want to delete this file in case I needed something from it later on....
*
*


// global var marking start time of round.
float g_roundStartTime=0;



void MessageBegin(int msg_dest, int msg_type, const float *pOrigin, edict_t *ed) {
	const char *name;
	//char msg[256];
	name=GET_USER_MSG_NAME(PLID, msg_type, NULL);
	if(!name) name="unknown";

	// "Countdown" message marks the start of a round.  We don't care when it ends
	if(strcmp(name,"Countdown") ==0) {
		UTIL_LogPrintf("+++ Countdown started +++\n");
		g_roundStartTime = gpGlobals->time;
	}


	RETURN_META(MRES_IGNORED);
}
*/

enginefuncs_t meta_engfuncs = {
	NULL,			// pfnPrecacheModel()
		NULL,			// pfnPrecacheSound()
		NULL,				// pfnSetModel()
		NULL,				// pfnModelIndex()
		NULL,			// pfnModelFrames()

		NULL,				// pfnSetSize()
		NULL,			// pfnChangeLevel()
		NULL,			// pfnGetSpawnParms()
		NULL,			// pfnSaveSpawnParms()

		NULL,				// pfnVecToYaw()
		NULL,			// pfnVecToAngles()
		NULL,			// pfnMoveToOrigin()
		NULL,				// pfnChangeYaw()
		NULL,			// pfnChangePitch()

		NULL,		// pfnFindEntityByString()
		NULL,			// pfnGetEntityIllum()
		NULL,		// pfnFindEntityInSphere()
		NULL,		// pfnFindClientInPVS()
		NULL,			// pfnEntitiesInPVS()

		NULL,			// pfnMakeVectors()
		NULL,			// pfnAngleVectors()

		NULL,			// pfnCreateEntity()
		NULL,			// pfnRemoveEntity()
		NULL,		// pfnCreateNamedEntity()

		NULL,				// pfnMakeStatic()
		NULL,			// pfnEntIsOnFloor()
		NULL,			// pfnDropToFloor()

		NULL,				// pfnWalkMove()
		NULL,				// pfnSetOrigin()

		NULL,				// pfnEmitSound()
		NULL,		// pfnEmitAmbientSound()

		NULL,				// pfnTraceLine()
		NULL,				// pfnTraceToss()
		NULL,		// pfnTraceMonsterHull()
		NULL,				// pfnTraceHull()
		NULL,				// pfnTraceModel()
		NULL,			// pfnTraceTexture()
		NULL,			// pfnTraceSphere()
		NULL,			// pfnGetAimVector()

		NULL,			// pfnServerCommand()
		NULL,			// pfnServerExecute()
		NULL,		// pfnClientCommand()	// d'NULL, ClientCommand in dllapi too

		NULL,			// pfnParticleEffect()
		NULL,				// pfnLightStyle()
		NULL,				// pfnDecalIndex()
		NULL,			// pfnPointContents()

		NULL, /*MessageBegin,*/			// pfnMessageBegin()
		NULL,				// pfnMessageEnd()

		NULL,				// pfnWriteByte()
		NULL,				// pfnWriteChar()
		NULL,				// pfnWriteShort()
		NULL,				// pfnWriteLong()
		NULL,				// pfnWriteAngle()
		NULL,				// pfnWriteCoord()
		NULL,			// pfnWriteString()
		NULL,			// pfnWriteEntity()

		NULL,			// pfnCVarRegister()
		NULL,			// pfnCVarGetFloat()
		NULL,			// pfnCVarGetString()
		NULL,			// pfnCVarSetFloat()
		NULL,			// pfnCVarSetString()

		NULL,			// pfnAlertMessage()
		NULL,			// pfnEngineFprintf()

		NULL,	// pfnPvAllocEntPrivateData()
		NULL,		// pfnPvEntPrivateData()
		NULL,		// pfnFreeEntPrivateData()

		NULL,			// pfnSzFromIndex()
		NULL,			// pfnAllocString()

		NULL, 			// pfnGetVarsOfEnt()
		NULL,		// pfnPEntityOfEntOffset()
		NULL,		// pfnEntOffsetOfPEntity()
		NULL,			// pfnIndexOfEdict()
		NULL,		// pfnPEntityOfEntIndex()
		NULL,		// pfnFindEntityByVars()
		NULL,			// pfnGetModelPtr()

		NULL,				// pfnRegUserMsg()

		NULL,		// pfnAnimationAutomove()
		NULL,		// pfnGetBonePosition()

		NULL,		// pfnFunctionFromName()
		NULL,		// pfnNameForFunction()

		NULL,			// pfnClientPrintf()			//! JOHN: engine callbacks so game DLL can print messages to individual clients
		NULL,			// pfnServerPrint()

		NULL,				// pfnCmd_Args()				//! these 3 added 
		NULL,				// pfnCmd_Argv()				//! so game DLL can easily 
		NULL,				// pfnCmd_Argc()				//! access client 'cmd' strings

		NULL,			// pfnGetAttachment()

		NULL,				// pfnCRC32_Init()
		NULL,	// pfnCRC32_ProcessBuffer()
		NULL,		// pfnCRC32_ProcessByte()
		NULL,			// pfnCRC32_Final()

		NULL,				// pfnRandomLong()
		NULL,			// pfnRandomFloat()

		NULL,				// pfnSetView()
		NULL,					// pfnTime()
		NULL,			// pfnCrosshairAngle()

		NULL,			// pfnLoadFileForMe()
		NULL,				// pfnFreeFile()

		NULL,				// pfnEndSection()				//! trigger_endsection
		NULL,		// pfnCompareFileTime()
		NULL,				// pfnGetGameDir()
		NULL,	// pfnCvar_RegisterVariable()
		NULL,		// pfnFadeClientVolume()
		NULL,		// pfnSetClientMaxspeed()
		NULL,		// pfnCreateFakeClient() 		//! returns NULL if fake client can't be created
		NULL,			// pfnRunPlayerMove()
		NULL,		// pfnNumberOfEntities()

		NULL,		// pfnGetInfoKeyBuffer()		//! passing in NULL gets the serverinfo
		NULL,			// pfnInfoKeyValue()
		NULL,			// pfnSetKeyValue()
		NULL,		// pfnSetClientKeyValue()

		NULL,				// pfnIsMapValid()
		NULL,			// pfnStaticDecal()
		NULL,		// pfnPrecacheGeneric()
		NULL, 		// pfnGetPlayerUserId()			//! returns the server assigned userid for this player.
		NULL,			// pfnBuildSoundMsg()
		NULL,		// pfnIsDedicatedServer()		//! is this a dedicated server?
		NULL,			// pfnCVarGetPointer()
		NULL,			// pfnGetPlayerWONId()			//! returns the server assigned WONid for this player.

		//! YWB 8/1/99 TFF Physics additions
		NULL,			// pfnInfo_RemoveKey()
		NULL,		// pfnGetPhysicsKeyValue()
		NULL,		// pfnSetPhysicsKeyValue()
		NULL,	// pfnGetPhysicsInfoString()
		NULL,			// pfnPrecacheEvent()
		NULL,			// pfnPlaybackEvent()

		NULL,				// pfnSetFatPVS()
		NULL,				// pfnSetFatPAS()

		NULL,		// pfnCheckVisibility()

		NULL,			// pfnDeltaSetField()
		NULL,		// pfnDeltaUnsetField()
		NULL,		// pfnDeltaAddEncoder()
		NULL,		// pfnGetCurrentPlayer()
		NULL,			// pfnCanSkipPlayer()
		NULL,			// pfnDeltaFindField()
		NULL,	// pfnDeltaSetFieldByIndex()
		NULL,	// pfnDeltaUnsetFieldByIndex()

		NULL,			// pfnSetGroupMask()

		NULL, // pfnCreateInstancedBaseline()		// d'NULL, CreateInstancedBaseline in dllapi too
		NULL,			// pfnCvar_DirectSet()

		NULL,		// pfnForceUnmodified()

		NULL,			// pfnGetPlayerStats()

		NULL,		// pfnAddServerCommand()

		// Added in SDK 2.2:
		NULL,	// pfnVoice_GetClientListening()
		NULL,	// pfnVoice_SetClientListening()

		// Added for HL 1109 (no SDK update):
		NULL,		// pfnGetPlayerAuthId()
// not recognized by my version
		// Added 2003/11/10 (no SDK update):
		//NULL,					// pfnSequenceGet()
		//NULL,			// pfnSequencePickSentence()
		//NULL,					// pfnGetFileSize()
		//NULL,			// pfnGetApproxWavePlayLen()
		//NULL,					// pfnIsCareerMatch()
		//NULL,		// pfnGetLocalizedStringLength()
		//NULL,		// pfnRegisterTutorMessageShown()
		//NULL,		// pfnGetTimesTutorMessageShown()
		//NULL,	// pfnProcessTutorMessageDecayBuffer()
		//NULL,	// pfnConstructTutorMessageDecayBuffer()
		//NULL,		// pfnResetTutorMessageDecayData()
		//
};

C_DLLEXPORT int GetEngineFunctions(enginefuncs_t *pengfuncsFromEngine, 
								   int *interfaceVersion) 
{
	if(!pengfuncsFromEngine) {
		LOG_ERROR(PLID, 
			"GetEngineFunctions called with null pengfuncsFromEngine");
		return(FALSE);
	}
	else if(*interfaceVersion != ENGINE_INTERFACE_VERSION) {
		LOG_ERROR(PLID, 
			"GetEngineFunctions version mismatch; requested=%d ours=%d", 
			*interfaceVersion, ENGINE_INTERFACE_VERSION);
		// Tell metamod what version we had, so it can figure out who is 
		// out of date.
		*interfaceVersion = ENGINE_INTERFACE_VERSION;
		return(FALSE);
	}
	memcpy(pengfuncsFromEngine, &meta_engfuncs, sizeof(enginefuncs_t));
	return TRUE;
}


