#include <extdll.h>

#include "osdep.h"              // win32 vsnprintf, etc
#include <cbase.h>
#include <enginecallback.h>     // ALERT()
#include "bod.h"

// Selected portions of dlls/util.cpp from SDK 2.1.
// Functions copied from there as needed...
// And modified to avoid buffer overflows (argh).

//=========================================================
// UTIL_LogPrintf - Prints a logged message to console.
// Preceded by LOG: ( timestamp ) < message >
//=========================================================
void UTIL_LogPrintf( char *fmt, ... )
{
  va_list argptr;
  static char string[1024];

  va_start(argptr, fmt);
  vsnprintf(string, sizeof(string), fmt, argptr);
  va_end(argptr);

  // Print to server console
  ALERT(at_logged, "%s", string);
}

CBaseEntity *UTIL_FindEntityByClassname( CBaseEntity *pStartEntity, const char *szName )
{
  return UTIL_FindEntityByString( pStartEntity, "classname", szName );
}

CBaseEntity *UTIL_FindEntityByString( CBaseEntity *pStartEntity, const char *szKeyword, const char *szValue )
{
  edict_t *pentEntity;

  if (pStartEntity)
   pentEntity = pStartEntity->edict();
  else
   pentEntity = NULL;

  pentEntity = FIND_ENTITY_BY_STRING( pentEntity, szKeyword, szValue );

  if (!FNullEnt(pentEntity))
   return CBaseEntity::Instance(pentEntity);
  return NULL;
}

CBaseEntity	*UTIL_PlayerByIndex( int playerIndex )
{
	CBaseEntity *pPlayer = NULL;

	if ( playerIndex > 0 && playerIndex <= gpGlobals->maxClients )
	{
		edict_t *pPlayerEdict = INDEXENT( playerIndex );
		if ( pPlayerEdict && !pPlayerEdict->free )
		{
			pPlayer = CBaseEntity::Instance( pPlayerEdict );	
		}
	}
	
	return pPlayer;
}

//Voogru stuff
BOOL UTIL_IsValidEntity( edict_t *pent )
{
	if ( !pent || pent->free || (pent->v.flags & FL_KILLME) )
		return FALSE;
	return TRUE;
}

void NSASayAll(const char *pText, int msg_type)
{
	if( CVAR_GET_FLOAT("nsa_disable_notice") > 0 )
		return;
	for (int i = 1; i <= gpGlobals->maxClients; i++ )
	{
		edict_t *pPlayer = g_engfuncs.pfnPEntityOfEntIndex( i );
		if (UTIL_IsValidEntity(pPlayer) && !FStrEq(STRING(pPlayer->v.netname),"") && FStrEq(STRING(pPlayer->v.classname),"player"))
		{
			ClientPrint2(pPlayer, msg_type, pText);
		}
	}
}

void ClientPrint2( edict_t *pEntity, int msg_dest, const char *msg_name)
{
	MESSAGE_BEGIN( MSG_ONE, REG_USER_MSG( "TextMsg", -1 ), NULL, pEntity );
	WRITE_BYTE( msg_dest );
        WRITE_STRING( msg_name );
        MESSAGE_END();
}

void TeamHudMessage(Team team, const hudtextparms_t &textparms, const char *pMessage)
{
	//LOG_MESSAGE(PLID,"First player of team %d", team);
	CBaseEntity *pPlayer = NULL;
	for ( int iIndex = 1; iIndex <= gpGlobals->maxClients; iIndex++ )
	{
		pPlayer = UTIL_PlayerByIndex( iIndex );
		if(pPlayer != NULL)
		{
			if(pPlayer->pev->team == team || team == ANY_TEAM)
				UTIL_HudMessage(pPlayer, textparms, pMessage);
		}
	}
}

void UTIL_HudMessage(CBaseEntity *pEntity, const hudtextparms_t &textparms, const char *pMessage)
{
	
	//if ( !pEntity || !pEntity->IsNetClient() ) //original version of line below that made plugin crash under linux
	if ( !pEntity )
		return;

	MESSAGE_BEGIN( MSG_ONE, SVC_TEMPENTITY, NULL, pEntity->pev ); 
		WRITE_BYTE( TE_TEXTMESSAGE );
		WRITE_BYTE( textparms.channel & 0xFF );

		WRITE_SHORT( FixedSigned16( textparms.x, 1<<13 ) );
		WRITE_SHORT( FixedSigned16( textparms.y, 1<<13 ) );
		WRITE_BYTE( textparms.effect );

		WRITE_BYTE( textparms.r1 );
		WRITE_BYTE( textparms.g1 );
		WRITE_BYTE( textparms.b1 );
		WRITE_BYTE( textparms.a1 );

		WRITE_BYTE( textparms.r2 );
		WRITE_BYTE( textparms.g2 );
		WRITE_BYTE( textparms.b2 );
		WRITE_BYTE( textparms.a2 );

		WRITE_SHORT( FixedUnsigned16( textparms.fadeinTime, 1<<8 ) );
		WRITE_SHORT( FixedUnsigned16( textparms.fadeoutTime, 1<<8 ) );
		WRITE_SHORT( FixedUnsigned16( textparms.holdTime, 1<<8 ) );

		if ( textparms.effect == 2 )
			WRITE_SHORT( FixedUnsigned16( textparms.fxTime, 1<<8 ) );

		const int MAX_MESSAGE_LENGTH = 488;
		if ( strlen( pMessage ) < MAX_MESSAGE_LENGTH )
		{
			WRITE_STRING( pMessage );
		}
		else
		{
			char tmp[MAX_MESSAGE_LENGTH];
			strncpy( tmp, pMessage, MAX_MESSAGE_LENGTH-1 );
			tmp[MAX_MESSAGE_LENGTH-1] = '\0';
			WRITE_STRING( tmp );
		}
	MESSAGE_END();
}

// short, flickering red on channel 1
void UTIL_HudMessageA(CBaseEntity *pEntity, const char *pMessage)
{
	hudtextparms_t hText;

	hText.channel = 1;
	hText.x = -1;
	hText.y = -1;
	hText.effect = 1;
	hText.r2 = 255;
	hText.g2 = 0;
	hText.b2 = 0;
	hText.a1 = 255;
	hText.r1 = 255;
	hText.g1 = 0;
	hText.b1 = 0;
	hText.a2 = 255;
	hText.fadeoutTime = 4.0;
	hText.fxTime = 4.0;

	UTIL_HudMessage(pEntity, hText, pMessage);
}

// eight second, top of screen on channel 2
void UTIL_HudMessageB(CBaseEntity *pEntity, const char *pMessage)
{
	hudtextparms_t hText;

	hText.channel = 2;
	hText.x = .40;
	hText.y = .10;
	hText.effect = 0;
	hText.r2 = 255;
	hText.g2 = 255;
	hText.b2 = 0;
	hText.a1 = 255;
	hText.r1 = 255;
	hText.g1 = 255;
	hText.b1 = 0;
	hText.a2 = 255;
	hText.fadeoutTime = 2.0;
	hText.holdTime = 8.0;
	hText.fxTime = 2.0;

	UTIL_HudMessage(pEntity, hText, pMessage);
}

/*
void UTIL_HudMessageAll_New(const hudtextparms_t &textparms, const char *pMessage)
{
	
	MESSAGE_BEGIN( MSG_ALL, SVC_TEMPENTITY, NULL ); 
		WRITE_BYTE( TE_TEXTMESSAGE );
		WRITE_BYTE( textparms.channel & 0xFF );

		WRITE_SHORT( FixedSigned16( textparms.x, 1<<13 ) );
		WRITE_SHORT( FixedSigned16( textparms.y, 1<<13 ) );
		WRITE_BYTE( textparms.effect );

		WRITE_BYTE( textparms.r1 );
		WRITE_BYTE( textparms.g1 );
		WRITE_BYTE( textparms.b1 );
		WRITE_BYTE( textparms.a1 );

		WRITE_BYTE( textparms.r2 );
		WRITE_BYTE( textparms.g2 );
		WRITE_BYTE( textparms.b2 );
		WRITE_BYTE( textparms.a2 );

		WRITE_SHORT( FixedUnsigned16( textparms.fadeinTime, 1<<8 ) );
		WRITE_SHORT( FixedUnsigned16( textparms.fadeoutTime, 1<<8 ) );
		WRITE_SHORT( FixedUnsigned16( textparms.holdTime, 1<<8 ) );

		if ( textparms.effect == 2 )
			WRITE_SHORT( FixedUnsigned16( textparms.fxTime, 1<<8 ) );

		if ( strlen( pMessage ) < 512 )
		{
			WRITE_STRING( pMessage );
		}
		else
		{
			char tmp[512];
			strncpy( tmp, pMessage, 511 );
			tmp[511] = 0;
			WRITE_STRING( tmp );
		}
	MESSAGE_END();
}
*/

bool UTIL_IsAdmin(edict_t *pEntity)
{
	// hmm - this is much easier.  It looks like someone didn't know much about the
	// standard C library?  OR -- wasn't using them for a reason??  
	// but FStrEq is a macro using strcmp, so we should be ok using it
	return strstr(CVAR_GET_STRING("mp_serverops"), GETPLAYERAUTHID(pEntity)) != NULL;

	/*
	cvar_t *serverops = ;
	unsigned int counter = 0;
	char outbuffer[200] = "";
	unsigned int outbuffer_position = 0;

	while(true)
	{
		if( serverops->string[counter] == ' ' ) 
		{
			++counter;
			continue;
		}
		if( serverops->string[counter] == ';' || serverops->string[counter] == '\0') 
		{
			outbuffer[outbuffer_position] = '\0';
			if(FStrEq(outbuffer, )) 
			{
				outbuffer[0] = '\0';
				return true;
			}
			outbuffer[0] = '\0';
			outbuffer_position = 0;
		}
		else 
		{
			outbuffer[outbuffer_position] = serverops->string[counter];
			++outbuffer_position;
		}
		++counter;
		if(serverops->string[counter-1] == '\0') 
		{
			return false;
		}
	}
	return false;
	*/
}


int UTIL_GetTeamCount(Team team)
{
	CBaseEntity* pPlayer;
	int iPlayers=0;
	for (int iIndex = 1; iIndex <= gpGlobals->maxClients; iIndex++)
	{
		pPlayer = UTIL_PlayerByIndex(iIndex);
		if(pPlayer != NULL)
		{
			if(pPlayer->pev->team == team)	iPlayers++;
		}
	}
	
	if(g_assignExtraPlayer == team)
		iPlayers++;

	return iPlayers;
}

const char* UTIL_GetPlayerName(edict_t* pEntity)
{
	if(!pEntity || pEntity->free)
		return "";

	return STRING(pEntity->v.netname);
}

Team UTIL_GetPlayerTeam(edict_t* pPlayerEdict)
{
	CBaseEntity* pPlayer = CBaseEntity::Instance( pPlayerEdict );

	if( !pPlayer ) {
		return NO_TEAM;
	} else {
		return intToTeam(pPlayer->pev->team);
	}
}

int UTIL_CheckTeamBalance(void)
{
	int iTeamMarines = UTIL_GetTeamCount(MARINE_TEAM);
	int iTeamAliens = UTIL_GetTeamCount(ALIEN_TEAM);

	if (abs(iTeamMarines - iTeamAliens) < UNBALANCED_PLAYER_DIFFERENCE)
		return NO_TEAM;


	if(iTeamMarines - iTeamAliens > UNBALANCED_PLAYER_DIFFERENCE-1)
	{
		return MARINE_TEAM;
	} else if(iTeamAliens - iTeamMarines > UNBALANCED_PLAYER_DIFFERENCE-1) {
		return ALIEN_TEAM;
	}

	return NO_TEAM; // but this shouldn't be reached...
}

CBaseEntity* UTIL_PlayerByName(const char* playerName)
{
	CBaseEntity* pPlayer = NULL;
	for (int i = 1; i <= gpGlobals->maxClients; i++ ) {
		pPlayer = UTIL_PlayerByIndex(i);
		if(!pPlayer)
			continue;
		if(strcmp(STRING(pPlayer->pev->netname), playerName)==0) {
			break;
		}
	}
	return pPlayer;
}

void UTIL_SendAdminsMessage(const char* msg)
{
	CBaseEntity *pPlayer = NULL;

	for(int playerIndex=1; playerIndex <= gpGlobals->maxClients; playerIndex++) {
		edict_t *pPlayerEdict = INDEXENT( playerIndex );
		if ( pPlayerEdict && !pPlayerEdict->free && UTIL_IsAdmin(pPlayerEdict))
		{
			pPlayer = CBaseEntity::Instance( pPlayerEdict );	
			UTIL_HudMessageA(pPlayer, msg);
		}
	}
}